{ <The SliderComponent>

  Copyright (C) <2020> <Bernd Hübner>
  Special Thanks to many members of the German Lazarus Forum
  Diverse Änderungen von Siro am 13.04.2020
  Invalidate für die Persistenten Blöcke eingeführt und Tastensteuerung eingeführt

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version with the following modification:

  As a special exception, the copyright holders of this library give you
  permission to link this library with independent modules to produce an
  executable, regardless of the license terms of these independent modules,and
  to copy and distribute the resulting executable under terms of your choice,
  provided that you also meet, for each linked independent module, the terms
  and conditions of the license of that module. An independent module is a
  module which is not derived from or based on this library. If you modify
  this library, you may extend this exception to your version of the library,
  but you are not obligated to do so. If you do not wish to do so, delete this
  exception statement from your version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1335, USA.
}
unit SliderComponent;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs,LCLIntf,
  LCLType, LMessages;


type
  TKnob = (ksCircle,ksRectangle,ksRoundrect,ksArrowUp,ksArrowDown,ksArrowLeft,KsArrowRight);
type
  TLook = (klNormal,klRipped);
type
  TOrientation = (scHorizontal,scVertical);

type
  TScaleStyle = (ssCircle,ssDash);
type
  TMultiKnob = class(TPersistent)
   private
    FOwner       : TCustomControl;   // Siro, hier wird der Besitzer gespeichert
    FShowKnob2   : boolean;          //2. Knopf sichtbar
    FShowKnob3   : boolean;          //3. Knopf sichtbar
    FColorKnob2  : TColor;           //2. Farbe
    FColorKnob3  : TColor;           //3. Farbe
    FPosKnob2    : integer;          //Position Knopf 2
    FPosKnob3    : integer;          //Position Knopf 3
   protected
    procedure SetShowKnob2(const Value:boolean);
    procedure SetShowKnob3(const Value:boolean);
    procedure SetColorKnob2(const aColor:TColor);
    procedure SetColorKnob3(const aColor:TColor);
    procedure SetPosKnob2(const aValue:integer);
    procedure SetPosKnob3(const aValue:integer);
   public
     constructor create(aOwner:TCustomControl);// Siro neuer Constructor mit Parameter des Besitzers
   published
    property ShowKnob2   : boolean read FShowKnob2   write SetShowKnob2  default false;
    property ShowKnob3   : boolean read FShowKnob3   write SetShowKnob3  default false;
    property ColorKnob2  : TColor  read FColorKnob2  write SetColorKnob2 default cllime;
    property ColorKnob3  : TColor  read FColorKnob3  write SetColorKnob3 default clblue;
    property PosKnob2    : integer read FPosKnob2    write SetPosKnob2   default 5;
    property PosKnob3    : integer read FPosKnob3    write SetPosKnob3   default 10;
  end;

type
  TChangeEvent = procedure(const Count: integer) of object;

type
  TIndicator = class(TPersistent)
   private
    FOwner              : TCustomControl;  //Siro, hier wird der Besitzer gespeichert
    FIndicatorVisible   : boolean;         //Wert Anzeige sichtbar
    FIndicatorColor     : TColor;          //Hintergrund Wertanzeige
    FIndicatorFontColor : TColor;          //Schriftfarbe
   protected
    procedure SetIndicatorVisible(const Value:boolean);
    procedure SetIndicatorColor(const aColor:TColor);
    procedure SetIndicatorFontColor(const aColor:TColor);
   public
     constructor create(aOwner:TCustomControl);    // Siro neuer Constructor mit Parameter des Besitzers
   published
    property IndicatorVisible   : boolean read FIndicatorVisible   write SetIndicatorVisible default false;
    property IndicatorColor     : TColor  read FIndicatorColor     write SetIndicatorColor default clMoneyGreen;
    property IndicatorFontColor : TColor  read FIndicatorFontColor write SetIndicatorFontColor default clblack;

  end;
type
  TMark = class(TPersistent)
   private
    FOwner              : TCustomControl;  //Siro, hier wird der Besitzer gespeichert
    FMarkVisible        : boolean;
    FMark2Visible       : boolean;
    FMarkColor          : TColor;
    FMarkPosition       : integer;
    FMark2Position      : integer;
    procedure SetMarkVisible(const Value:boolean);
    procedure SetMarkColor(const aColor:TColor);
    procedure SetMarkPosition(const aValue:integer);
    procedure SetMark2Visible(const Value:boolean);
    procedure SetMark2Position(const aValue:integer);
   public
    constructor create(aOwner:TCustomControl);  // Siro neuer Constructor mit Parameter des Besitzers
   published
    property MarkVisible        : boolean read FMarkVisible             write SetMarkVisible default false;
    property MarkColor          : TColor  read FMarkColor               write SetMarkColor default clNavy;
    property MarkPosition       : integer read FMarkPosition            write SetMarkPosition default 0;
    property Mark2Visible       : boolean read FMark2Visible            write SetMark2Visible default false;
    property Mark2Position      : integer read FMark2Position           write SetMark2Position;
  end;
type
  TScale = class(TPersistent)
   private
    FOwner           : TCustomControl;  // Siro, hier wird der Besitzer gespeichert
    FLineColor       : TColor;
    FMarkColor       : TColor;
    FInterval        : integer;
    FShowScale       : boolean;
    FOtherSide       : boolean;
    FBigMarkColor    : TColor;
    FBigMarkInterval : integer;
    FShowBigMark     : boolean;
    FBigMarkSize     : double;
    FScaleStyle      : TScaleStyle;
    procedure SetLineColor(const aColor:TColor);
    procedure SetMarkColor(const aColor:TColor);
    procedure SetInterval(const aValue:integer);
    procedure SetShowScale(const Value:boolean);
    procedure SetOtherSide(const Value:boolean);
    procedure SetBigMarkColor(const aColor:TColor);
    procedure SetBigMarkInterval(const aValue:integer);
    procedure SetShowBigMark(const Value:boolean);
    procedure SetBigMarkSize(const aValue:double);
    procedure SetScaleStyle(const aStyle:TScaleStyle);
   public
    constructor create(aOwner:TCustomControl);  // Siro neuer Constructor mit Parameter des Besitzers
    property BigMarkSize  : double Read FBigMarkSize  Write SetBigMarkSize;
   published
    property LineColor : TColor  Read FLineColor Write SetLineColor default clblack;
    property MarkColor : TColor  Read FMarkColor Write SetMarkColor default clnavy;
    property Interval  : integer Read FInterval  Write SetInterval default 2;
    property ShowScale : boolean Read FShowScale Write SetShowScale default false;
    property OtherSide : boolean Read FOtherSide Write SetOtherSide default false;
    property BigMarkColor     : TColor  Read FBigMarkColor     Write SetBigMarkColor default clnavy;
    property BigMarkInterval  : integer Read FBigMarkInterval  Write SetBigMarkInterval default 5;
    property ShowBigMark      : boolean Read FShowBigMark      Write SetShowBigMark default false;
    property ScaleStyle: TScaleStyle    Read FScaleStyle       Write SetScaleStyle default ssCircle;
  end;
type
  TTrack = class(TPersistent)
   private
    FOwner                : TCustomControl;
    FTrackColor           : TColor;       //Schlitzfarbe
    FTrackColorSel        : TColor;       //Schlitzfarbe selektiert
    FTrackColorSelVisible : boolean;      //Selektiertefarbe ein
    FTrackColorExt        : TColor;       //Schlitzfarbe extra
    FTrackColorExtVisible : boolean;      //Extrafarbe ein
    FExtMin               : integer;      //Min Wert
    FExtMax               : integer;      //Max Wert
    procedure SetTrackColor(const aColor:TColor);
    procedure SetTrackColorSelVisible(const Value:boolean);
    procedure SetTrackColorSel(const aColor:TColor);
    procedure SetTrackColorExtVisible(const Value:boolean);
    procedure SetTrackColorExt(const aColor:TColor);
    procedure SetExtMin(const aValue:integer);
    procedure SetExtMax(const aValue:integer);
   public
    constructor create(aOwner:TCustomControl);
   published
   property TrackColor         : TColor  read FTrackColor             write SetTrackColor default clWhite;
   property SelColor           : TColor  read FTrackColorSel          write SetTrackColorSel default clGray;
   property ShowSelRange       : boolean read FTrackColorSelVisible   write SetTrackColorSelVisible default true;
   property ExtraColor         : TColor  read FTrackColorExt          write SetTrackColorExt default clred;
   property ShowExtraColor     : boolean read FTrackColorExtVisible   write SetTrackColorExtVisible default false;
   property ExtraRangeMin      : integer read FExtMin                 write SetExtMin default 0;
   property ExtraRangeMax      : integer read FExtMax                 write SetExtMax default 0;

end;
type

 { TSliderComponent }

 TSliderComponent = class (TCustomControl)  //  frueher (TGraphicControl)  Siro 13.04.2020
  private
    { Private declarations }
    FKnob                : TKnob;        //SchieberKnopf
    FLook                : TLook;        //Knopf Aussehen
    FLookColor           : TColor;       //Stiftfarbe vom Knopf für Look
    FBackgroundColor     : TColor;       //Hintergrundfarbe
    FKnobColor           : TColor;       //SchiebeKnopffarbe
    FBorderColor         : TColor;       //Randfarbe
    FBorderWidth         : integer;      //Randdicke
    FMin                 : integer;      //kleinster Wert
    FMax                 : integer;      //größter Wert
    FStep                : integer;      //Schritt bei Klick
    FPosition            : integer;      //Startwert und Position
    FIndicator           : TIndicator;   //Wertanzeige
    FMark                : TMark;        //Markierung
    FOnChange            : TChangeEvent; //Geändert Ereigniss
    FScale               : TScale;       //Skala
    FMultiKnob           : TMultiKnob;   //Mehrere Knöpfe
    FFocusColor1         : TColor;       //1.Farbe des Focusrahmens
    FFocusColor2         : TColor;       //2.Farbe des Focusrahmens
    FOrientation         : TOrientation; //Ausrichtung des Schiebers
    FTrack               : TTrack;       //Spur,Schlitz,Track
    FReversed            : boolean;      // MinMax umgekehrt
    procedure SetKnob(const aValue : TKnob);
    procedure SetKnobColor(const aColor:TColor);
    procedure SetKnobLook(const aValue : TLook);
    procedure SetLookColor(const aColor:TColor);
    procedure SetBackgroundColor(const aColor:TColor);
    procedure SetBorderColor(const aColor:TColor);
    procedure SetBorderWidth(const aValue:integer);
    procedure SetMin(const aValue:integer);
    procedure SetMax(const aValue:integer);
    procedure SetStep(const aValue:integer);
    procedure setPosition(const aValue:integer);
    procedure SetFocusColor1(const aColor:TColor);
    procedure SetFocusColor2(const aColor:TColor);
    procedure SetOrientation(const aValue : TOrientation);
    procedure SetReversed(const Value : boolean);
    procedure CalculateDimensions;                   //Berechnung der Dimensionen
    procedure DrawRectangle;                         //zeichnet das Rechteck
    function  CalculateKnob(aPos: integer): TRect;   //Knopf Größe und Position berechnen
    function  PixelSteps:double;                     //Intervall des Knopfes in Pixel
    procedure DrawTrackColorsHorizontal;             //Extrafarbe und SelFarbe
    procedure DrawTrackColorsVertical;
    procedure DrawScaleHorizontal;                   //zeichnet die Skala
    procedure DrawScaleVertical;
    procedure DrawMarkHorizontal;                    //zeichnet die Markierungen
    procedure DrawMarkVertical;
    procedure BuildTheKnob(Idx:integer);             //Knopf machen
    procedure DrawIndicator;                         //Wertanzeige
  protected
    { Protected declarations,geschützte Deklarationen }
    scaling        : double;                 //Skalierungsfaktor
    CenterY        : double;                 //Mitte in der Höhe
    TrLe,TrTo      : integer;                //Track left top
    TrRe,TrBo      : integer;                //Track right bottom
    TrackLenght    : integer;                //Länge Track
    Knob           : array [1..3] of TRect;  //Knöpfe
    MKnob          : integer;                //Knopfmitte
    KIdx           : integer;                //Knopf Index
    offset         : integer;                //Versatz der Position
    slide          : boolean;                //Knopf wird geschoben
    Arrow1         : array [0..2] of TPoint; //PfeilKnopf
    Arrow2         : array [0..4] of TPoint;
    Mark1          : array [0..2] of TPoint; //Markierung
    Mark2          : array [0..2] of TPoint;
    procedure Loaded; override;
    procedure mousemove(Shift: TShiftState; X, Y: Integer);override;
    procedure MouseDown(Button: TMouseButton;Shift: TShiftState; X, Y: Integer);override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure CNKeyDown    (var Message: TLMKeyDown);    message CN_KEYDOWN;  // Siro
    procedure DoExit;  override;    // wenn die Komponente den Focus verliert 13.04.2020 Siro
    procedure DoEnter; override;    // wenn die Komponente den Focus erhält   13.04.2020 Siro
  public
    { Public declarations, öffentliche Deklarationen, nicht sichtbar im OI }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    property Step                : integer read FStep           write SetStep default 1;
    property FocusColor1         : TColor  read FFocusColor1    write SetFocusColor1 default clblue;
    property FocusColor2         : TColor  read FFocusColor2    write SetFocusColor2 default clblack;
    property KnobLookColor       : TColor  read FLookColor      write SetLookColor default clsilver;
  published
    { Published declarations, veröffentlichte Deklarationen, sichtbar im OI }
   property KnobStyle          : TKnob   read fKnob                          write SetKnob default kscircle;
   property KnobLook           : TLook   read FLook                          write SetKnobLook default klNormal;
   property BackgroundColor    : TColor  read FBackgroundColor               write SetBackgroundColor default clMenuBar;
   property KnobColor          : TColor  read FKnobColor                     write SetKnobColor default clblack;
   property BorderColor        : TColor  read FBorderColor                   write SetBorderColor default clgray;
   property BorderWidth        : integer read FBorderWidth                   write SetBorderWidth default 1;
   property RangeMin           : integer read FMin                           write SetMin default 0;
   property RangeMax           : integer read FMax                           write SetMax default 10;
   property Position           : integer read FPosition                      write setPosition default 0;
   property OnChange           : TChangeEvent read FOnChange                 write FOnChange;
   property Scale              : TScale  read FScale                         write FScale;
   property Mark               : TMark   read FMark                          write FMark;
   property Indicator          : TIndicator   read FIndicator                write FIndicator;
   property MultiKnob          : TMultiKnob   read FMultiKnob                write FMultiKnob;
   property Orientation        : TOrientation read FOrientation              write SetOrientation default scHorizontal;
   property Track              : TTrack  read FTrack                         write FTrack;
   property Reversed           : boolean read FReversed                      write SetReversed default false;
   property TabStop default TRUE;   // 13.03.2020 Siro
end;
var
   Pos   : array [1..3] of integer; //Position der 3 Knöpfe

procedure Register;

implementation

constructor TSliderComponent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  width                 := 200;
  height                :=  30;
  if FOrientation = scVertical then
   begin
    width                 :=  30;
    height                := 200;
   end;
  FKnob                 := kscircle;
  FBackgroundColor      := clMenuBar;
  FKnobColor            := clblack;
  FBorderColor          := clgray;
  FBorderWidth          :=   1;
  FMin                  :=   0;
  FMax                  :=  10;
  FStep                 :=   1;
  FPosition             :=   0;
  FFocusColor1          := clblue;
  FFocusColor2          := clblack;
  FOrientation          := scHorizontal;
  FReversed             := false;
  FLook                 := klNormal;
  FLookColor            := clsilver;

  FScale := TScale.Create(self);
  FScale.FLineColor      := clblack;
  FScale.FMarkColor      := clnavy;
  FScale.FInterval       := 2;
  FScale.FShowScale      := false;
  FScale.FOtherSide      := false;
  FScale.FBigMarkColor   := clNavy;
  FScale.FBigMarkInterval:= 5;
  FScale.FShowBigMark    := false;
  FScale.FBigMarkSize    := 1.5;
  FScale.FScaleStyle     := ssCircle;

  FMark                 := TMark.Create(self);     // Siro, Besitzer übergeben ist die Komponenten selber
  FMark.FMarkVisible    := false;
  FMark.FMarkColor      := clNavy;
  FMark.FMarkPosition   := 0;
  FMark.FMark2Visible   := false;
  FMark.FMark2Position  := 0;

  FIndicator                       := TIndicator.Create(self);  // Siro, Besitzer ist die Komponente selber
  FIndicator.FIndicatorVisible     := false;
  FIndicator.FIndicatorColor       := clMoneyGreen;
  FIndicator.FIndicatorFontColor   := clblack;

  FMultiKnob                := TMultiKnob.Create(self);     // Siro, Besitzer ist die Komponente selber
  FMultiKnob.FShowKnob2     := false;
  FMultiKnob.FShowKnob3     := false;
  FMultiKnob.FColorKnob2    := cllime;
  FMultiKnob.FColorKnob3    := clblue;
  FMultiKnob.FPosKnob2      :=  5;
  FMultiKnob.FPosKnob3      := 10;

  FTrack                       := TTrack.create(self);
  FTrack.FTrackColor            := clWhite;
  FTrack.FTrackColorSel         := clGray;
  FTrack.FTrackColorSelVisible  := true;
  FTrack.FTrackColorExt         := clred;
  FTrack.FTrackColorExtVisible  := false;
  FTrack.FExtMin               := 0;
  FTrack.FExtMax               := 0;

  slide:= false;
  TabStop:=TRUE;    // 13.04.2020 Siro
end;

destructor TSliderComponent.Destroy;
begin
 FScale.Free;
 FMark.Free;
 FIndicator.Free;
 FMultiKnob.Free;
 FTrack.Free;
 inherited Destroy;
end;

procedure TSliderComponent.SetFocusColor1(const aColor:TColor);
begin
 if FFocusColor1=aColor then exit;
 FFocusColor1:=aColor;
 Invalidate
end;

procedure TSliderComponent.SetFocusColor2(const aColor:TColor);
begin
 if FFocusColor2=aColor then exit;
 FFocusColor2:=aColor;
 Invalidate
end;


procedure TSliderComponent.SetOrientation(const aValue : TOrientation);
var wi,he : integer;
begin
 if FOrientation = aValue then exit;
 FOrientation := aValue;
 if width<height then
  begin
   wi := height;
   he := width;
  end else
  begin
   wi:= width;
   he:= height;
  end;
 if FOrientation = scVertical then
  begin
   width := he;
   height:= wi;
  end else
  begin
   width := wi;
   height:= he;
  end;
 Invalidate;
end;

procedure TSliderComponent.SetReversed(const Value:boolean);
begin
 if FReversed=Value then exit;
 FReversed:=Value;
 Invalidate
end;


procedure TSliderComponent.SetKnob(const aValue : TKnob);
begin
 if fKnob <> aValue then
 begin
  fKnob := aValue;
  Invalidate;
 end;
end;

procedure TSliderComponent.SetKnobLook(const aValue : TLook);
begin
 if fLook <> aValue then
 begin
  fLook := aValue;
  Invalidate;
 end;
end;

procedure TSliderComponent.SetLookColor(const aColor : TColor);
begin
 if fLookColor <> aColor then
 begin
  fLookColor := aColor;
  Invalidate;
 end;
end;

procedure TSliderComponent.SetBackgroundColor(const aColor:TColor);
begin
 if aColor = FBackgroundColor then exit;
 FBackgroundColor:=aColor;
 Invalidate;
end;


procedure TSliderComponent.SetKnobColor(const aColor:TColor);
begin
 if aColor = FKnobColor then exit;
 FKnobColor:=aColor;
 Invalidate;
end;

procedure TSliderComponent.SetBorderColor(const aColor:TColor);
begin
 if aColor = FBorderColor then exit;
 FBorderColor:=aColor;
 Invalidate;
end;

procedure TSliderComponent.SetBorderWidth(const aValue:integer);
begin
 if aValue = FBorderWidth then exit;
 FBorderWidth:=aValue;
 Invalidate;
end;

procedure TSliderComponent.SetMin(const aValue:integer);
begin
 if aValue = FMin then exit;
 if aValue > FMax then exit;
 FMin:=aValue;
 Invalidate;
end;

procedure TSliderComponent.SetMax(const aValue:integer);
begin
 if aValue = FMax then exit;
 if aValue < FMin then exit;
 FMax:=aValue;
 Invalidate;
end;

procedure TSliderComponent.SetStep(const aValue:integer);
begin
 if aValue = FStep then exit;
 FStep:=aValue;
 Invalidate;
end;

procedure TSliderComponent.setPosition(const aValue:integer);
begin
 if aValue = FPosition then exit;
 FPosition:=aValue;
 if FPosition < FMin then FPosition := FMin;
 if FPosition > FMax then FPosition := FMax;
 Pos[1]:=FPosition;
 Invalidate;
end;

//xxxxxxxxxxxxxxxxxxxxxxxxxx---AB HIER TSCALE---XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

constructor TScale.create(aOwner:TCustomControl);
begin
 FOwner:=aOwner;
 inherited create;  // Siro
end;

procedure TScale.SetLineColor(const aColor:TColor);
begin
 if aColor = FLineColor then exit;
 FLineColor:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TScale.SetMarkColor(const aColor:TColor);
begin
 if aColor = FMarkColor then exit;
 FMarkColor:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TScale.SetInterval(const aValue:integer);
begin
 if aValue = FInterval then exit;
 FInterval:=AValue;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TScale.SetShowScale(const Value:boolean);
begin
 if Value = FShowScale then exit;
 FShowScale := Value;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TScale.SetOtherSide(const Value:boolean);
begin
 if Value = FOtherSide then exit;
 FOtherSide := Value;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TScale.SetBigMarkColor(const aColor:TColor);
begin
 if aColor = FBigMarkColor then exit;
 FBigMarkColor:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TScale.SetBigMarkInterval(const aValue:integer);
begin
 if aValue = FBigMarkInterval then exit;
 FBigMarkInterval:=AValue;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TScale.SetShowBigMark(const Value:boolean);
begin
 if Value = FShowBigMark then exit;
 FShowBigMark := Value;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TScale.SetBigMarkSize(const aValue:double);
begin
 if aValue = FBigMarkSize then exit;
 FBigMarkSize:=AValue;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TScale.SetScaleStyle(const aStyle:TScaleStyle);
begin
 if aStyle = FScaleStyle then exit;
 FScaleStyle:=aStyle;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX---AB HIER TTRACK---XXXXXXXXXXXXXXXXXXXXXXXXXXXXX

constructor TTrack.create(aOwner:TCustomControl);
begin
 FOwner:=aOwner;
 inherited create;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TTrack.SetTrackColor(const aColor:TColor);
begin
 if aColor = FTrackColor then exit;
 FTrackColor:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TTrack.SetTrackColorSelVisible(const Value:boolean);
begin
 if FTrackColorSelVisible = Value then exit;
 FTrackColorSelVisible   := Value;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TTrack.SetTrackColorSel(const aColor:TColor);
begin
 if aColor = FTrackColorSel then exit;
 FTrackColorSel:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TTrack.SetTrackColorExtVisible(const Value:boolean);
begin
 if FTrackColorExtVisible = Value then exit;
 FTrackColorExtVisible   := Value;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TTrack.SetTrackColorExt(const aColor:TColor);
begin
 if aColor = FTrackColorExt then exit;
 FTrackColorExt:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TTrack.SetExtMin(const aValue:integer);
begin
 if FExtMin = aValue then exit;
 FExtMin:=aValue;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TTrack.SetExtMax(const aValue:integer);
begin
 if FExtMax = aValue then exit;
 FExtMax:=aValue;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX---AB HIER TINDICATOR---XXXXXXXXXXXXXXXXXXXXXXXXX

constructor TIndicator.create(aOwner:TCustomControl);
begin
 FOwner:=aOwner;
 inherited create;  // Siro
end;

procedure TIndicator.SetIndicatorVisible(const Value:boolean);
begin
 if FIndicatorVisible =Value then exit;
 FIndicatorVisible :=Value;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TIndicator.SetIndicatorColor(const aColor:TColor);
begin
 if FIndicatorColor=aColor then exit;
 FIndicatorColor:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TIndicator.SetIndicatorFontColor(const aColor:TColor);
begin
 if FIndicatorFontColor=aColor then exit;
 FIndicatorFontColor:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX---AB HIER TMARK---XXXXXXXXXXXXXXXXXXXXXXXXXX

constructor TMark.create(aOwner:TCustomControl);
begin
 FOwner:=aOwner;
 inherited create;  // Siro
end;

procedure TMark.SetMarkVisible(const Value:boolean);
begin
 if FMarkVisible =Value then exit;
 FMarkVisible :=Value;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TMark.SetMarkColor(const aColor:TColor);
begin
 if FMarkColor=aColor then exit;
 FMarkColor:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TMark.SetMarkPosition(const aValue:integer);
begin
 if FMarkPosition = aValue then exit;
 FMarkPosition := aValue;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TMark.SetMark2Visible(const Value:boolean);
begin
 if FMark2Visible = Value then exit;
 FMark2Visible :=Value;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TMark.SetMark2Position(const aValue:integer);
begin
 if FMark2Position = aValue then exit;
 FMark2Position := aValue;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX---AB HIER TMULTIKNOB---XXXXXXXXXXXXXXXXXXXXXXX

constructor TMultiKnob.create(aOwner:TCustomControl);
begin
 FOwner:=aOwner;
 inherited create;  //Siro
end;

procedure TMultiKnob.SetShowKnob2(const Value:boolean);
begin
 if fShowKnob2 = Value then exit;
 fShowKnob2 :=Value;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TMultiKnob.SetShowKnob3(const Value:boolean);
begin
 if fShowKnob3 = Value then exit;
 fShowKnob3 :=Value;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TMultiKnob.SetColorKnob2(const aColor:TColor);
begin
 if fColorKnob2=aColor then exit;
 fColorKnob2:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TMultiKnob.SetColorKnob3(const aColor:TColor);
begin
 if fColorKnob3=aColor then exit;
 fColorKnob3:=aColor;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TMultiKnob.SetPosKnob2(const aValue:integer);
begin
 if FPosKnob2 = aValue then exit;
 FPosKnob2 := aValue;
 Pos[2]:=FPosKnob2;
 if Assigned(FOwner) then FOwner.Invalidate;
end;

procedure TMultiKnob.SetPosKnob3(const aValue:integer);
begin
 if FPosKnob3 = aValue then exit;
 FPosKnob3 := aValue;
 Pos[3]:=FPosKnob3;
 if Assigned(FOwner) then FOwner.Invalidate;
end;


{XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX----ENDE DER SET PROZEDUREN----XXXXXXXXXXXX}
//  --- SIRO Tastensteuerung eingeführt ----------------------------------------
procedure TSliderComponent.CNKeyDown(var Message: TLMKeyDown);
var Variant1 : boolean = false;
    Variant2 : boolean = false;
    Variant3 : boolean = false;
    Variant4 : boolean = false;
begin

 if ((FOrientation = scHorizontal) and (not Reversed)) then Variant1:=true;
 if ((FOrientation = scHorizontal) and (Reversed))     then Variant2:=true;
 if ((FOrientation = scVertical)   and (not Reversed)) then Variant3:=true;
 if ((FOrientation = scVertical)   and (Reversed))     then Variant4:=true;
  begin
   with Message do begin
    Result := 1;
    case CharCode of
        VK_Left  : begin
                    if Variant1 then if Position > RangeMin then Position:=Position-1;
                    if Variant2 then if Position < RangeMax then Position:=Position+1;
                    end;
        VK_RIGHT : begin
                    if Variant2 then if Position > RangeMin then Position:=Position-1;
                    if Variant1 then if Position < RangeMax then Position:=Position+1;
                    end;
        VK_UP    : begin
                    if Variant3 then if Position > RangeMin then Position:=Position-1;
                    if Variant4 then if Position < RangeMax then Position:=Position+1;
                    end;
        VK_DOWN  : begin
                    if Variant4 then if Position > RangeMin then Position:=Position-1;
                    if Variant3 then if Position < RangeMax then Position:=Position+1;
                    end;
        VK_HOME  : Position:=0;
        VK_END   : Position:=RangeMax;
    else begin
        Result := 0;
      end;
    end;
   end;
  end;
if Assigned(OnChange) then OnChange(Position);
Variant1:=false;Variant2:=false;Variant3:=false;Variant4:=false;
inherited; // Vorgaenger immer aufrufen, jedoch mit dem entsprechendem Result
end;

procedure TSliderComponent.MouseDown(Button: TMouseButton;Shift: TShiftState; X, Y: Integer);
var P1        : TPoint;
    A,B,C,D,E,F,G,H : integer;
procedure SwapOrientation;
begin
 A:=0;B:=0;C:=width;D:=Knob[1].Top;
 E:=0;F:=Knob[1].Bottom;G:=width;H:=height;
end;

begin
 SetFocus;  // Siro
 P1.X:=x;
 P1.Y:=y;

 if FMultiKnob.FShowKnob3 then
   begin
    if ptinrect(Knob[3],P1) then
     begin
      slide:=true;
      KIdx:=3;
     end;
   end;
 if FMultiKnob.FShowKnob2 then
   begin
    if ptinrect(Knob[2],P1) then
     begin
      slide:=true;
      KIdx:=2;
     end;
   end;
 if ptinrect(Knob[1],P1) then
     begin
      slide:=true;
      KIdx:=1;
     end;
 //Ab hier Maussteps
 A:=0;B:=0;C:=Knob[1].Left;D:=height;
 E:=Knob[1].Right;F:=0;G:=width;H:=height;
 if FOrientation = scVertical then SwapOrientation;
 if not slide then
   begin
    if ptinrect(rect(A,B,C,D),P1) then
     begin
      if not FReversed then Position:= Position - FStep;
      if FReversed then Position:= Position + FStep;
      Invalidate;
     end;
    if ptinrect(rect(E,F,G,H),P1) then
     begin
      if not FReversed then Position:= Position + FStep;
      if FReversed then Position:= Position - FStep;
      Invalidate;
     end;
   if Assigned(OnChange) then OnChange(Position);
   end;
end;

procedure TSliderComponent.mousemove(Shift: TShiftState; X, Y: Integer);
var lv,w,Tr1,Tr2:integer;
procedure SwapOrientation;
 begin
  lv:=x; x :=y; y :=lv; w:=height;
  Tr1:=TrTo;Tr2:=TrBo;
 end;
begin
 w:=width;Tr1:=TrLe;Tr2:=TrRe;
 if FOrientation = scVertical then SwapOrientation;
 if slide then
  begin
   for lv:= FMin to FMax do
    begin
     if not Reversed then
      begin
       if (x>=Tr1+round(PixelSteps*(lv-offset))) and (x<Tr1+round(PixelSteps*(lv+1-offset))) then Pos[KIdx]:=lv;
       if x > w-round(scaling*8) then Pos[KIdx]:=FMax;
       if x < round(scaling*8) then Pos[KIdx]:=FMin;
      end else
      begin
       if (x<=Tr2-round(PixelSteps*(lv-offset))) and (x>Tr2-round(PixelSteps*(lv+1-offset))) then Pos[KIdx]:=lv;
       if x > w-round(scaling*8) then Pos[KIdx]:=FMin;
       if x < round(scaling*8) then Pos[KIdx]:=FMax;
      end;
    end;
   Position:=Pos[1];
   FMultiKnob.FPosKnob2:=Pos[2];
   FMultiKnob.FPosKnob3:=Pos[3];
   if Assigned(OnChange) then OnChange(Position);
   Invalidate;
  end;
end;

procedure TSliderComponent.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
 if slide  then slide  :=false;
end;


procedure TSliderComponent.DoEnter; // erhält den Focus 13.04.2020 Siro
begin
  invalidate;  // Neu zeichnen mit Focus
  inherited;
end;

procedure TSliderComponent.DoExit; // verliert den Focus 13.04.2020 Siro
begin
  invalidate;  // Neu zeichnen, jetzt ohne Focus
  inherited;
end;

procedure TSliderComponent.Loaded;
var temp : integer;
begin
  inherited;
 if FMin > FMax then
  begin
   temp:=FMin;
   FMin:=FMax;
   FMax:=temp;
  end;
 if Position<FMin then Position := FMin;
 if Position>FMax then Position := FMax;
end;

{XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX----AB HIER ZEICHNEN----XXXXXXXXXXXXXXXXXXXXXXXXX}
procedure TSliderComponent.CalculateDimensions;
begin
 if FOrientation = scHorizontal then
  begin
   scaling := height / 30;      // Skalierungsfaktor
   CenterY := height / 2;       // Mitte in der Höhe
   TrLe    := round((scaling*8)+(6*scaling));         //Track left
   TrTo    := round(CenterY-(3*scaling));             //Track top
   TrRe    := width-round(((scaling*8)+(6*scaling))); //Track right
   TrBo    := round(CenterY+(3*scaling));             //Track bottom
   TrackLenght  := TrRe-TrLe;                         //Länge Track
  end else
  begin
   scaling := width / 30;
   CenterY := width / 2;
   TrLe    := round(CenterY-(3*scaling));              //Track left
   TrTo    := round((scaling*8)+(6*scaling));          //Track top
   TrRe    := round(CenterY+(3*scaling));              //Track right
   TrBo    := height-round(((scaling*8)+(6*scaling))); //Track bottom
   TrackLenght  := TrBo-TrTo;                          //Länge Track
  end;

end;

function TSliderComponent.CalculateKnob(aPos:integer):TRect;
var lv : integer;
begin
 offset:=(0+FMin);
//Mitte des Knopfes bestimmen
 if FOrientation = scHorizontal then
 begin
  for lv:= FMin to FMax do
   begin
    if not Reversed then
     begin
      if lv= aPos then MKnob:= TrLe+round(PixelSteps*(lv-offset));
     end
    else  //Reversed
     begin
      if lv= aPos then MKnob:= TrRe-round(PixelSteps*(lv-offset));
     end;
   end;
   Result:= rect((round(MKnob-(6*scaling)))-1,(round(CenterY-(6*scaling)))-1,
               (round(MKnob+(6*scaling)))+1,(round(CenterY+(6*scaling)))+1);
  end else   //scVertical
  begin
   for lv:= FMin to FMax do
    begin
     if not Reversed then
      begin
       if lv= aPos then MKnob:= TrTo+round(PixelSteps*(lv-offset));
      end
     else //Reversed
      begin
       if lv= aPos then MKnob:= TrBo-round(PixelSteps*(lv-offset));
      end;
    end;
   Result:= rect((round(CenterY-(6*scaling)))-1,(round(MKnob-(6*scaling)))-1,
                 (round(CenterY+(6*scaling)))+1,(round(MKnob+(6*scaling)))+1);
  end;

end;

function  TSliderComponent.PixelSteps:double;
var interval : integer;
begin
 {Positiverbereich 0 bis ..}
 if FMin >=0 then
  begin
   interval:= (FMax-FMin);
  end;
 {Negativerbereich }
 if FMax <=0 then
  begin
   interval:= (FMax-FMin);
  end;
 {Um 0 }
 if (FMin <0)  and (FMax>0) then
  begin
   interval:= (0-FMin)+FMax;
  end;
 Result:= TrackLenght/interval;
end;

procedure TSliderComponent.DrawRectangle;
var tmpi : integer;

begin
//Hintergrund
 tmpi              := canvas.Pen.Width;
 canvas.Pen.Width  := FBorderWidth;
 canvas.Pen.Color  := FBorderColor;
 canvas.Brush.Color:=FBackgroundColor;
 canvas.Rectangle(0,0,Width,Height);
 canvas.Pen.Width  := tmpi;
//Schiebenut,Track
 canvas.Pen.Color  := FTrack.FTrackColor;
 canvas.Brush.Color:= FTrack.FTrackColor;
 canvas.Rectangle(TrLe,TrTo,TrRe,TrBo);

end;

procedure TSliderComponent.DrawTrackColorsHorizontal;
var lv,ExtL,ExtR : integer;
begin
 if FTrack.FTrackColorExtVisible then         //Extrafarbe
  begin
   canvas.Pen.Color  := FTrack.FTrackColorExt;
   canvas.Brush.Color:= FTrack.FTrackColorExt;

   for lv:= FMin to FMax do
    begin
     if not Reversed then
      begin
       if lv= FTrack.FExtMin then ExtL:= TrLe+round(PixelSteps*(lv-offset));
       if lv= FTrack.FExtMax then ExtR:= TrLe+round(PixelSteps*(lv-offset));
      end  else
      begin
       if lv= FTrack.FExtMin then ExtR:= TrRe-round(PixelSteps*(lv-offset));
       if lv= FTrack.FExtMax then ExtL:= TrRe-round(PixelSteps*(lv-offset));
      end;
    end;
   canvas.Rectangle(ExtL,TrTo,ExtR,TrBo);
 end;
 if FTrack.FTrackColorSelVisible then      //SelectedColor
  begin
   canvas.Pen.Color  := FTrack.FTrackColorSel;
   canvas.Brush.Color:= FTrack.FTrackColorSel;
   if not Reversed then
    begin
     if (FMax<=0) then canvas.Rectangle(MKnob,TrTo,TrRe,TrBo)
     else canvas.Rectangle(TrLe,TrTo,MKnob,TRBo);
    end else
    begin
     if (FMax<=0) then canvas.Rectangle(TrLe,TrTo,MKnob,TrBo)
     else canvas.Rectangle(MKnob,TrTo,TrRe,TrBo);
    end;
  end;
end;

procedure TSliderComponent.DrawTrackColorsVertical;
var lv,ExtT,ExtB : integer;
begin
 if FTrack.FTrackColorExtVisible then           //Extrafarbe
  begin
   canvas.Pen.Color  := FTrack.FTrackColorExt;
   canvas.Brush.Color:= FTrack.FTrackColorExt;

   for lv:= FMin to FMax do
    begin
     if not Reversed then
      begin
       if lv= FTrack.FExtMin then ExtT:= TrTo+round(PixelSteps*(lv-offset));
       if lv= FTrack.FExtMax then ExtB:= TrTo+round(PixelSteps*(lv-offset));
      end else
      begin
       if lv= FTrack.FExtMin then ExtB:= TrBo-round(PixelSteps*(lv-offset));
       if lv= FTrack.FExtMax then ExtT:= TrBo-round(PixelSteps*(lv-offset));
      end;
    end;
   canvas.Rectangle(TrLe,ExtT,TrRe,ExtB);
  end;

 if FTrack.FTrackColorSelVisible then        //SelectedColor
  begin
   canvas.Pen.Color  := FTrack.FTrackColorSel;
   canvas.Brush.Color:= FTrack.FTrackColorSel;
   if not Reversed then
    begin
     if (FMax<=0) then canvas.Rectangle(TrLe,MKnob,TrRe,TrBo)
     else canvas.Rectangle(TrLe,TrTo,TrRe,MKnob);
    end else
    begin
     if (FMax<=0) then canvas.Rectangle(TrLe,TrTo,TrRe,MKnob)
     else canvas.Rectangle(TrLe,MKnob,TrRe,TrBo);
    end;
  end;
end;

procedure TSliderComponent.DrawScaleHorizontal;
var ScMark : TRect;
    bigfak : double;
    lv     : integer;
begin
 if not FScale.FShowScale then exit;
   canvas.Pen.Color   :=FScale.FLineColor;
   if not FScale.FOtherSide then
   canvas.Line(TrLe,Knob[1].Bottom+round(3*scaling),TrRe,Knob[1].Bottom+round(3*scaling))
   else
   canvas.Line(TrLe,Knob[1].Top-round(3*scaling),TrRe,Knob[1].Top-round(3*scaling));

   canvas.Pen.Color   :=FScale.FMarkColor;    //kleine Markierungen
   canvas.Brush.Color :=FScale.FMarkColor;
   if not FScale.OtherSide then
    begin
     if FScale.FScaleStyle = ssCircle then
      begin
       ScMark             := rect(TrLe-round(2*scaling),(Knob[1].Bottom+round(3*scaling))-round(2*scaling),TrLe+round(2*scaling),(Knob[1].Bottom+round(3*scaling))+round(2*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FInterval)-1 do
       Canvas.Ellipse(SCMark.Left+round(PixelSteps*lv*FScale.FInterval),SCMark.Top,SCMark.Right+round(PixelSteps*lv*FScale.FInterval),SCMark.Bottom);
       Canvas.Ellipse((SCMark.Left+TrRe)-TrLe,SCMark.Top,(SCMark.Right+TrRe)-TrLe,SCMark.Bottom);
      end;
     if FScale.FScaleStyle = ssDash then
      begin
       ScMark             := rect(TrLe,(Knob[1].Bottom+round(3*scaling))-round(2*scaling),TrLe,(Knob[1].Bottom+round(3*scaling))+round(2*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FInterval)-1 do
       Canvas.Line(SCMark.Left+round(PixelSteps*lv*FScale.FInterval),SCMark.Top,SCMark.Right+round(PixelSteps*lv*FScale.FInterval),SCMark.Bottom);
       Canvas.Line((SCMark.Left+TrRe)-TrLe,SCMark.Top,(SCMark.Right+TrRe)-TrLe,SCMark.Bottom);
      end;
    end else //OtherSide
    begin
     if FScale.FScaleStyle = ssCircle then
      begin
       ScMark := rect(TrLe-round(2*scaling),(Knob[1].Top-round(3*scaling))-round(2*scaling),TrLe+round(2*scaling),(Knob[1].Top-round(3*scaling))+round(2*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FInterval)-1 do
       Canvas.Ellipse(SCMark.Left+round(PixelSteps*lv*FScale.FInterval),SCMark.Top,SCMark.Right+round(PixelSteps*lv*FScale.FInterval),SCMark.Bottom);
       Canvas.Ellipse((SCMark.Left+TrRe)-TrLe,SCMark.Top,(SCMark.Right+TrRe)-TrLe,SCMark.Bottom);
      end;
      if FScale.FScaleStyle = ssDash then
      begin
       ScMark := rect(TrLe,(Knob[1].Top-round(3*scaling))-round(2*scaling),TrLe,(Knob[1].Top-round(3*scaling))+round(2*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FInterval)-1 do
       Canvas.Line(SCMark.Left+round(PixelSteps*lv*FScale.FInterval),SCMark.Top,SCMark.Right+round(PixelSteps*lv*FScale.FInterval),SCMark.Bottom);
       Canvas.Line((SCMark.Left+TrRe)-TrLe,SCMark.Top,(SCMark.Right+TrRe)-TrLe,SCMark.Bottom);
      end;
    end;

 if FScale.FShowBigMark then
  begin
   bigfak:= 2*FScale.FBigMarkSize;
   canvas.Pen.Color   :=FScale.FBigMarkColor;    //Big Marks
   canvas.Brush.Color :=FScale.FBigMarkColor;
   if not FScale.OtherSide then
    begin
     if FScale.FScaleStyle = ssCircle then
      begin
       ScMark             := rect(TrLe-round(bigfak*scaling),(Knob[1].Bottom+round(3*scaling))-round(bigfak*scaling),TrLe+round(bigfak*scaling),(Knob[1].Bottom+round(3*scaling))+round(bigfak*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FBigMarkInterval)-1 do
       Canvas.Ellipse(SCMark.Left+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Top,SCMark.Right+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Bottom);
       Canvas.Ellipse((SCMark.Left+TrRe)-TrLe,SCMark.Top,(SCMark.Right+TrRe)-TrLe,SCMark.Bottom);
      end;
      if FScale.FScaleStyle = ssDash then
      begin
       ScMark             := rect(TrLe,(Knob[1].Bottom+round(3*scaling))-round(bigfak*scaling),TrLe,(Knob[1].Bottom+round(3*scaling))+round(bigfak*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FBigMarkInterval)-1 do
       Canvas.Line(SCMark.Left+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Top,SCMark.Right+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Bottom);
       Canvas.Line((SCMark.Left+TrRe)-TrLe,SCMark.Top,(SCMark.Right+TrRe)-TrLe,SCMark.Bottom);
      end;
    end else
    begin
     if FScale.FScaleStyle = ssCircle then
      begin
       ScMark := rect(TrLe-round(bigfak*scaling),(Knob[1].Top-round(3*scaling))-round(bigfak*scaling),TrLe+round(bigfak*scaling),(Knob[1].Top-round(3*scaling))+round(bigfak*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FBigMarkInterval)-1 do
       Canvas.Ellipse(SCMark.Left+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Top,SCMark.Right+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Bottom);
       Canvas.Ellipse((SCMark.Left+TrRe)-TrLe,SCMark.Top,(SCMark.Right+TrRe)-TrLe,SCMark.Bottom);
      end;
      if FScale.FScaleStyle = ssDash then
      begin
       ScMark := rect(TrLe,(Knob[1].Top-round(3*scaling))-round(bigfak*scaling),TrLe,(Knob[1].Top-round(3*scaling))+round(bigfak*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FBigMarkInterval)-1 do
       Canvas.Line(SCMark.Left+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Top,SCMark.Right+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Bottom);
       Canvas.Line((SCMark.Left+TrRe)-TrLe,SCMark.Top,(SCMark.Right+TrRe)-TrLe,SCMark.Bottom);
      end;
    end;
  end;
end;

procedure TSliderComponent.DrawScaleVertical;
var ScMark : TRect;
    bigfak : double;
    lv     : integer;
begin
 if not FScale.FShowScale then exit;
   canvas.Pen.Color   :=FScale.FLineColor;
   if not FScale.OtherSide then
    canvas.Line(Knob[1].Left-round(3*scaling),TrTo,Knob[1].Left-round(3*scaling),TrBo)
   else
    canvas.Line(Knob[1].Right+round(3*scaling),TrTo,Knob[1].Right+round(3*scaling),TrBo);


   canvas.Pen.Color   :=FScale.FMarkColor;     //kleine Markierungen
   canvas.Brush.Color :=FScale.FMarkColor;
   if not FScale.OtherSide then
    begin
     if FScale.FScaleStyle = ssCircle then
      begin
       ScMark             := rect(((Knob[1].Left-round(3*scaling))+round(2*scaling)),TrTo-round(2*scaling),(Knob[1].Left-round(3*scaling))-round(2*scaling),TrTo+round(2*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FInterval)-1 do
       Canvas.Ellipse(SCMark.Left,SCMark.Top+round(PixelSteps*lv*FScale.FInterval),SCMark.Right,SCMark.Bottom+round(PixelSteps*lv*FScale.FInterval));
       Canvas.Ellipse(SCMark.Left,(SCMark.Top+TrBo)-TrTo,SCMark.Right,(SCMark.Bottom+TrBo)-TrTo);
      end;
     if FScale.FScaleStyle = ssDash then
      begin
       ScMark             := rect(((Knob[1].Left-round(3*scaling))+round(2*scaling)),TrTo,(Knob[1].Left-round(3*scaling))-round(2*scaling),TrTo);
       for lv:= 0 to round((FMax-FMin)/FScale.FInterval)-1 do
       Canvas.Line(SCMark.Left,SCMark.Top+round(PixelSteps*lv*FScale.FInterval),SCMark.Right,SCMark.Bottom+round(PixelSteps*lv*FScale.FInterval));
       Canvas.Line(SCMark.Left,(SCMark.Top+TrBo)-TrTo,SCMark.Right,(SCMark.Bottom+TrBo)-TrTo);
      end;

    end else
    begin
     if FScale.FScaleStyle = ssCircle then
      begin
       ScMark := rect(((Knob[1].Right+round(3*scaling))+round(2*scaling)),TrTO-round(2*scaling),(Knob[1].Right+round(3*scaling))-round(2*scaling),TrTo+round(2*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FInterval)-1 do
       Canvas.Ellipse(SCMark.Left,SCMark.Top+round(PixelSteps*lv*FScale.FInterval),SCMark.Right,SCMark.Bottom+round(PixelSteps*lv*FScale.FInterval));
       Canvas.Ellipse(SCMark.Left,(SCMark.Top+TrBo)-TrTo,SCMark.Right,(SCMark.Bottom+TrBo)-TrTo);
      end;
     if FScale.FScaleStyle = ssDash then
      begin
       ScMark := rect(((Knob[1].Right+round(3*scaling))+round(2*scaling)),TrTO,(Knob[1].Right+round(3*scaling))-round(2*scaling),TrTo);
       for lv:= 0 to round((FMax-FMin)/FScale.FInterval)-1 do
       Canvas.Line(SCMark.Left,SCMark.Top+round(PixelSteps*lv*FScale.FInterval),SCMark.Right,SCMark.Bottom+round(PixelSteps*lv*FScale.FInterval));
       Canvas.Line(SCMark.Left,(SCMark.Top+TrBo)-TrTo,SCMark.Right,(SCMark.Bottom+TrBo)-TrTo);
      end;
    end;
 if FScale.FShowBigMark then
  begin
   bigfak:= 2*FScale.FBigMarkSize;
   canvas.Pen.Color   :=FScale.FBigMarkColor;     //Big Marks
   canvas.Brush.Color :=FScale.FBigMarkColor;
   if not FScale.OtherSide then
    begin
     if FScale.FScaleStyle = ssCircle then
      begin
       ScMark := rect(((Knob[1].Left-round(3*scaling))+round(bigfak*scaling)),TrTo-round(bigfak*scaling),(Knob[1].Left-round(3*scaling))-round(bigfak*scaling),TrTo+round(bigfak*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FBigMarkInterval)-1 do
       Canvas.Ellipse(SCMark.Left,SCMark.Top+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Right,SCMark.Bottom+round(PixelSteps*lv*FScale.FBigMarkInterval));
       Canvas.Ellipse(SCMark.Left,(SCMark.Top+TrBo)-TrTo,SCMark.Right,(SCMark.Bottom+TrBo)-TrTo);
      end;
     if FScale.FScaleStyle = ssDash then
      begin
       ScMark := rect(((Knob[1].Left-round(3*scaling))+round(bigfak*scaling)),TrTo,(Knob[1].Left-round(3*scaling))-round(bigfak*scaling),TrTo);
       for lv:= 0 to round((FMax-FMin)/FScale.FBigMarkInterval)-1 do
       Canvas.Line(SCMark.Left,SCMark.Top+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Right,SCMark.Bottom+round(PixelSteps*lv*FScale.FBigMarkInterval));
       Canvas.Line(SCMark.Left,(SCMark.Top+TrBo)-TrTo,SCMark.Right,(SCMark.Bottom+TrBo)-TrTo);
      end;
    end else
    begin
     if FScale.FScaleStyle = ssCircle then
      begin
       ScMark := rect(((Knob[1].Right+round(3*scaling))+round(bigfak*scaling)),TrTo-round(bigfak*scaling),(Knob[1].Right+round(3*scaling))-round(bigfak*scaling),TrTo+round(bigfak*scaling));
       for lv:= 0 to round((FMax-FMin)/FScale.FBigMarkInterval)-1 do
       Canvas.Ellipse(SCMark.Left,SCMark.Top+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Right,SCMark.Bottom+round(PixelSteps*lv*FScale.FBigMarkInterval));
       Canvas.Ellipse(SCMark.Left,(SCMark.Top+TrBo)-TrTo,SCMark.Right,(SCMark.Bottom+TrBo)-TrTo);
      end;
     if FScale.FScaleStyle = ssDash then
      begin
       ScMark := rect(((Knob[1].Right+round(3*scaling))+round(bigfak*scaling)),TrTo,(Knob[1].Right+round(3*scaling))-round(bigfak*scaling),TrTo);
       for lv:= 0 to round((FMax-FMin)/FScale.FBigMarkInterval)-1 do
       Canvas.Line(SCMark.Left,SCMark.Top+round(PixelSteps*lv*FScale.FBigMarkInterval),SCMark.Right,SCMark.Bottom+round(PixelSteps*lv*FScale.FBigMarkInterval));
       Canvas.Line(SCMark.Left,(SCMark.Top+TrBo)-TrTo,SCMark.Right,(SCMark.Bottom+TrBo)-TrTo);
      end;
    end;
   end;
end;

procedure TSliderComponent.DrawMarkHorizontal;
var lv : integer;
begin
 if (not FMark.FMarkVisible) and (not FMark.FMark2Visible) then exit;
 //Position der Markierung
 for lv:= FMin to FMax do
   begin
    if lv= FMark.FMarkPosition then
     begin
      if not Reversed then
       begin
        Mark1[0]:=Point(TrLe+round(Pixelsteps*(lv-offset)),Knob[1].Bottom);
        Mark1[1]:=Point(Mark1[0].X+round(5*scaling),Knob[1].Bottom+round(5*scaling));
        Mark1[2]:=Point(Mark1[0].X-round(5*scaling),Knob[1].Bottom+round(5*scaling));
       end else
       begin
        Mark1[0]:=Point(TrRe-round(Pixelsteps*(lv-offset)),Knob[1].Bottom);
        Mark1[1]:=Point(Mark1[0].X+round(5*scaling),Knob[1].Bottom+round(5*scaling));
        Mark1[2]:=Point(Mark1[0].X-round(5*scaling),Knob[1].Bottom+round(5*scaling))
       end;
     end;
   end;
 for lv:= FMin to FMax do         //Position der 2.Markierung
   begin
    if lv= FMark.FMark2Position then
     begin
      if not Reversed then
       begin
        Mark2[0]:=Point(TrLe+round(Pixelsteps*(lv-offset)),Knob[1].Bottom);
        Mark2[1]:=Point(Mark2[0].X+round(5*scaling),Knob[1].Bottom+round(5*scaling));
        Mark2[2]:=Point(Mark2[0].X-round(5*scaling),Knob[1].Bottom+round(5*scaling));
       end else
       begin
        Mark2[0]:=Point(TrRe-round(Pixelsteps*(lv-offset)),Knob[1].Bottom);
        Mark2[1]:=Point(Mark2[0].X+round(5*scaling),Knob[1].Bottom+round(5*scaling));
        Mark2[2]:=Point(Mark2[0].X-round(5*scaling),Knob[1].Bottom+round(5*scaling));
       end;
     end;
   end;
 if FMark.FMarkVisible then    //Markierung
  begin
   canvas.Pen.Color  := FMark.FMarkColor;
   canvas.Brush.Color:= FMark.FMarkColor;
   canvas.Polygon(Mark1);
  end;
 if FMark.FMark2Visible then    //2.Markierung
  begin
   canvas.Pen.Color  := FMark.FMarkColor;
   canvas.Brush.Color:= FMark.FMarkColor;
   canvas.Polygon(Mark2);
  end;
end;

procedure TSliderComponent.DrawMarkVertical;
var lv : integer;
begin
 if (not FMark.FMarkVisible) and (not FMark.FMark2Visible) then exit;
 //Position der Markierung
 for lv:= FMin to FMax do
   begin
    if lv= FMark.FMarkPosition then
     begin
      if not Reversed then
       begin
        Mark1[0]:=Point(Knob[1].Left,TrTo+round(Pixelsteps*(lv-offset)));
        Mark1[1]:=Point(Knob[1].Left-round(5*scaling),Mark1[0].y+round(5*scaling));
        Mark1[2]:=Point(Knob[1].Left-round(5*scaling),Mark1[0].Y-round(5*scaling));
       end else
       begin
        Mark1[0]:=Point(Knob[1].Left,TrBo-round(Pixelsteps*(lv-offset)));
        Mark1[1]:=Point(Knob[1].Left-round(5*scaling),Mark1[0].y+round(5*scaling));
        Mark1[2]:=Point(Knob[1].Left-round(5*scaling),Mark1[0].Y-round(5*scaling));
       end;
     end;
   end;
 for lv:= FMin to FMax do         //Position der 2.Markierung
   begin
    if lv= FMark.FMark2Position then
     begin
      if not Reversed then
       begin
        Mark2[0]:=Point(Knob[1].Left,TrTo+round(Pixelsteps*(lv-offset)));
        Mark2[1]:=Point(Knob[1].Left-round(5*scaling),Mark2[0].y+round(5*scaling));
        Mark2[2]:=Point(Knob[1].Left-round(5*scaling),Mark2[0].Y-round(5*scaling));
       end else
       begin
        Mark2[0]:=Point(Knob[1].Left,TrBo-round(Pixelsteps*(lv-offset)));
        Mark2[1]:=Point(Knob[1].Left-round(5*scaling),Mark2[0].y+round(5*scaling));
        Mark2[2]:=Point(Knob[1].Left-round(5*scaling),Mark2[0].Y-round(5*scaling));
       end;
     end;
   end;
 if FMark.FMarkVisible then    //Markierung
  begin
   canvas.Pen.Color  := FMark.FMarkColor;
   canvas.Brush.Color:= FMark.FMarkColor;
   canvas.Polygon(Mark1);
  end;
 if FMark.FMark2Visible then    //2.Markierung
  begin
   canvas.Pen.Color  := FMark.FMarkColor;
   canvas.Brush.Color:= FMark.FMarkColor;
   canvas.Polygon(Mark2);
  end;

end;

procedure TSliderComponent.BuildTheKnob(Idx: integer);
begin
 canvas.Pen.Width:=1;
 case Idx of
  1: begin
      if FLook = klNormal then canvas.Pen.Color  := FKnobColor
      else canvas.Pen.Color  := FLookColor;
      canvas.Brush.Color:= FKnobColor;
     end;
  2: begin
      if not FMultiKnob.FShowKnob2 then exit;
      if FLook = klNormal then canvas.Pen.Color  := FMultiKnob.ColorKnob2
      else canvas.Pen.Color  := FLookColor;
      canvas.Brush.Color:= FMultiKnob.ColorKnob2;
     end;
  3: begin
      if not FMultiKnob.FShowKnob3 then exit;
      if FLook = klNormal then canvas.Pen.Color  := FMultiKnob.ColorKnob3
      else canvas.Pen.Color  := FLookColor;
      canvas.Brush.Color:= FMultiKnob.ColorKnob3;
     end;
 end;

 if FKnob = ksrectangle then
  begin
   if FLook = klNormal then canvas.Rectangle(Knob[Idx]);
   if FLook = klRipped then
    begin
     canvas.Rectangle(Knob[Idx]);
     canvas.Rectangle(Knob[Idx].Left+round(3*scaling),Knob[Idx].Top+round(3*scaling),Knob[Idx].Right-round(3*scaling),Knob[Idx].Bottom-round(3*scaling));
     if FOrientation = scHorizontal then
      begin
       canvas.line(Knob[Idx].Left+round(5*scaling),Knob[Idx].Top+round(5*scaling),Knob[Idx].Left+round(5*scaling),Knob[Idx].Bottom-round(5*scaling));
       canvas.line(Knob[Idx].Right-round(5*scaling),Knob[Idx].Top+round(5*scaling),Knob[Idx].Right-round(5*scaling),Knob[Idx].Bottom-round(5*scaling));
      end else
      begin
       canvas.line(Knob[Idx].Left+round(5*scaling),Knob[Idx].Top+round(5*scaling),Knob[Idx].Right-round(5*scaling),Knob[Idx].Top+round(5*scaling));
       canvas.line(Knob[Idx].Left+round(5*scaling),Knob[Idx].Bottom-round(5*scaling),Knob[Idx].Right-round(5*scaling),Knob[Idx].Bottom-round(5*scaling));
      end;
    end;
  end;
 if FKnob = kscircle    then
  begin
   if FLook = klNormal then canvas.Ellipse(Knob[Idx]);
   if FLook = klRipped then
    begin
     canvas.Ellipse(Knob[Idx]);
     canvas.Ellipse(Knob[Idx].Left+round(3*scaling),Knob[Idx].Top+round(3*scaling),Knob[Idx].Right-round(3*scaling),Knob[Idx].Bottom-round(3*scaling));
    end;
  end;
 if FKnob = ksroundrect then
  begin
   if FLook = klNormal then canvas.RoundRect(Knob[Idx],round(4*scaling),round(4*scaling));
   if FLook = klRipped then
    begin
     canvas.RoundRect(Knob[Idx],round(4*scaling),round(4*scaling));
     canvas.RoundRect(rect(Knob[Idx].Left+round(3*scaling),Knob[Idx].Top+round(3*scaling),Knob[Idx].Right-round(3*scaling),Knob[Idx].Bottom-round(3*scaling)),round(4*scaling),round(4*scaling));
     if FOrientation = scHorizontal then
      begin
       canvas.line(Knob[Idx].Left+round(5*scaling),Knob[Idx].Top+round(5*scaling),Knob[Idx].Left+round(5*scaling),Knob[Idx].Bottom-round(5*scaling));
       canvas.line(Knob[Idx].Right-round(5*scaling),Knob[Idx].Top+round(5*scaling),Knob[Idx].Right-round(5*scaling),Knob[Idx].Bottom-round(5*scaling));
      end else
      begin
       canvas.line(Knob[Idx].Left+round(5*scaling),Knob[Idx].Top+round(5*scaling),Knob[Idx].Right-round(5*scaling),Knob[Idx].Top+round(5*scaling));
       canvas.line(Knob[Idx].Left+round(5*scaling),Knob[Idx].Bottom-round(5*scaling),Knob[Idx].Right-round(5*scaling),Knob[Idx].Bottom-round(5*scaling));
      end;
    end;
  end;
 if (FKnob = ksArrowUp) or (FKnob = ksArrowRight) then
  begin
   if (FLook = klNormal) and (FOrientation = scHorizontal) then
    begin
     canvas.Rectangle(Rect(Knob[Idx].Left,Knob[Idx].Top,Knob[Idx].Right,Knob[Idx].Bottom));
     Arrow1[0].Y := Knob[Idx].Top;
     Arrow1[0].X := Knob[Idx].Left;
     Arrow1[1].X := MKnob;
     Arrow1[1].Y := Knob[Idx].Top-(round(Knob[Idx].Width/4));
     Arrow1[2].X := Knob[Idx].Right-1;
     Arrow1[2].Y := Knob[Idx].Top;
     canvas.Polygon(Arrow1);
    end;
   if (FLook = klNormal) and (FOrientation = scVertical)  then
     begin
      canvas.Rectangle(Rect(Knob[Idx].Left,Knob[Idx].Top,Knob[Idx].Right,Knob[Idx].Bottom));
      Arrow1[0].Y := Knob[Idx].Top;
      Arrow1[0].X := Knob[Idx].Right;
      Arrow1[1].Y := MKnob;
      Arrow1[1].X := Knob[Idx].Right+(round(Knob[Idx].Width/4));
      Arrow1[2].X := Knob[Idx].Right;
      Arrow1[2].Y := Knob[Idx].Bottom-1;
      canvas.Polygon(Arrow1);
     end;
   if (FLook = klRipped) and (FOrientation = scHorizontal) then
    begin
     Arrow2[0].Y := Knob[Idx].Top;
     Arrow2[0].X := Knob[Idx].Left;
     Arrow2[1].X := MKnob;
     Arrow2[1].Y := Knob[Idx].Top-(round(Knob[Idx].Width/4));
     Arrow2[2].X := Knob[Idx].Right;
     Arrow2[2].Y := Knob[Idx].Top;
     Arrow2[3].X := Knob[Idx].Right;
     Arrow2[3].Y := Knob[Idx].Bottom;
     Arrow2[4].X := Knob[Idx].Left;
     Arrow2[4].Y := Knob[Idx].Bottom;

     canvas.Polygon(Arrow2);
     canvas.Line(Knob[Idx].Left+round(5*scaling),Knob[Idx].Top+round(1.2*scaling),Knob[Idx].Left+round(5*scaling),Knob[Idx].Bottom-round(5*scaling));
     canvas.Line(Knob[Idx].Right-round(5*scaling),Knob[Idx].Top+round(1.2*scaling),Knob[Idx].Right-round(5*scaling),Knob[Idx].Bottom-round(5*scaling));
    end;
    if (FLook = klRipped) and (FOrientation = scVertical)  then
     begin
      Arrow2[0].Y := Knob[Idx].Top;
      Arrow2[0].X := Knob[Idx].Right;
      Arrow2[1].Y := MKnob;
      Arrow2[1].X := Knob[Idx].Right+(round(Knob[Idx].Width/4));
      Arrow2[2].X := Knob[Idx].Right;
      Arrow2[2].Y := Knob[Idx].Bottom;
      Arrow2[3].X := Knob[Idx].Left;
      Arrow2[3].Y := Knob[Idx].Bottom;
      Arrow2[4].X := Knob[Idx].Left;
      Arrow2[4].Y := Knob[Idx].Top;

      canvas.Polygon(Arrow2);
      canvas.Line(Knob[Idx].Left+round(3*scaling),Knob[Idx].Top+round(5*scaling),Knob[Idx].Right-round(1.2*scaling),Knob[Idx].Top+round(5*scaling));
      canvas.Line(Knob[Idx].Left+round(3*scaling),Knob[Idx].Bottom-round(5*scaling),Knob[Idx].Right-round(1.2*scaling),Knob[Idx].Bottom-round(5*scaling));
     end;
 end;
 if (FKnob = ksArrowDown) or (FKnob = ksArrowLeft) then
  begin
   if (FLook = klNormal) and (FOrientation = scHorizontal) then
   begin
    canvas.Rectangle(Rect(Knob[Idx].Left,Knob[Idx].Top,Knob[Idx].Right,Knob[Idx].Bottom));
    Arrow1[0].Y := Knob[Idx].Bottom;
    Arrow1[0].X := Knob[Idx].Left;
    Arrow1[1].X := MKnob;
    Arrow1[1].Y := Knob[Idx].Bottom+(round(Knob[Idx].Width/4));
    Arrow1[2].X := Knob[Idx].Right-1;
    Arrow1[2].Y := Knob[Idx].Bottom;
    canvas.Polygon(Arrow1);
   end;
   if (FLook = klNormal) and (FOrientation = scVertical) then
    begin
     canvas.Rectangle(Rect(Knob[Idx].Left,Knob[Idx].Top,Knob[Idx].Right,Knob[Idx].Bottom));
     Arrow1[0].Y := Knob[Idx].Top;
     Arrow1[0].X := Knob[Idx].Left;
     Arrow1[1].Y := MKnob;
     Arrow1[1].X := Knob[Idx].Left-(round(Knob[Idx].Width/4));
     Arrow1[2].X := Knob[Idx].Left;
     Arrow1[2].Y := Knob[Idx].Bottom-1;
     canvas.Polygon(Arrow1);
    end;
   if (FLook = klRipped) and (FOrientation = scHorizontal) then
    begin
     Arrow2[0].Y := Knob[Idx].Bottom;
     Arrow2[0].X := Knob[Idx].Left;
     Arrow2[1].X := MKnob;
     Arrow2[1].Y := Knob[Idx].Bottom+(round(Knob[Idx].Width/4));
     Arrow2[2].X := Knob[Idx].Right;
     Arrow2[2].Y := Knob[Idx].Bottom;
     Arrow2[3].X := Knob[Idx].Right;
     Arrow2[3].Y := Knob[Idx].Top;
     Arrow2[4].X := Knob[Idx].Left;
     Arrow2[4].Y := Knob[Idx].Top;

     canvas.Polygon(Arrow2);
     canvas.Line(Knob[Idx].Left+round(5*scaling),Knob[Idx].Top+round(5*scaling),Knob[Idx].Left+round(5*scaling),Knob[Idx].Bottom-round(1.2*scaling));
     canvas.Line(Knob[Idx].Right-round(5*scaling),Knob[Idx].Top+round(5*scaling),Knob[Idx].Right-round(5*scaling),Knob[Idx].Bottom-round(1.2*scaling));
    end;
   if (FLook = klRipped) and (FOrientation = scVertical) then
    begin
     Arrow2[0].Y := Knob[Idx].Top;
     Arrow2[0].X := Knob[Idx].Left;
     Arrow2[1].Y := MKnob;
     Arrow2[1].X := Knob[Idx].Left-(round(Knob[Idx].Width/4));
     Arrow2[2].X := Knob[Idx].Left;
     Arrow2[2].Y := Knob[Idx].Bottom;
     Arrow2[3].X := Knob[Idx].Right;
     Arrow2[3].Y := Knob[Idx].Bottom;
     Arrow2[4].X := Knob[Idx].Right;
     Arrow2[4].Y := Knob[Idx].Top;

     canvas.Polygon(Arrow2);
     canvas.Line(Knob[Idx].Left+round(1.2*scaling),Knob[Idx].Top+round(5*scaling),Knob[Idx].Right-round(3*scaling),Knob[Idx].Top+round(5*scaling));
     canvas.Line(Knob[Idx].Left+round(1.2*scaling),Knob[Idx].Bottom-round(5*scaling),Knob[Idx].Right-round(3*scaling),Knob[Idx].Bottom-round(5*scaling));
    end;
 end;
end;

procedure TSliderComponent.DrawIndicator;
var temp : integer;
begin
 canvas.Brush.Color   := FIndicator.FIndicatorColor;
 canvas.Pen.Color     := FIndicator.FIndicatorColor;
 canvas.Font.Height   := round(10*scaling);
 if canvas.Font.Height < 10 then canvas.Font.Height:= 10;
 canvas.Font.Color    := FIndicator.FIndicatorFontColor;
 if FOrientation = scHorizontal then
  begin
   if fPosition <= round(fmax/2) then
    begin
     if not Reversed then
      if (FKnob = ksArrowDown) or (FKnob = ksArrowUp) or (FKnob = ksArrowLeft) or (FKnob = ksArrowRight)
      then  canvas.TextOut(Knob[1].Right+1,round(centerY)-round(canvas.TextHeight(inttostr(fPosition))/2),inttostr(fPosition))
      else canvas.TextOut(Knob[1].Right,round(centerY)-round(canvas.TextHeight(inttostr(fPosition))/2),inttostr(fPosition))
     else
      canvas.TextOut(Knob[1].Left-canvas.TextWidth(inttostr(fPosition)),round(centerY)-round(canvas.TextHeight(inttostr(fPosition))/2),inttostr(fPosition));
     end;
   if fPosition > round(fmax/2) then
    begin
     if not Reversed then
      canvas.TextOut(Knob[1].Left-canvas.TextWidth(inttostr(fPosition)),round(centerY)-round(canvas.TextHeight(inttostr(fPosition))/2),inttostr(fPosition))
     else
      if (FKnob = ksArrowDown) or (FKnob = ksArrowUp) or (FKnob = ksArrowLeft) or (FKnob = ksArrowRight)
      then canvas.TextOut(Knob[1].Right+1,round(centerY)-round(canvas.TextHeight(inttostr(fPosition))/2),inttostr(fPosition))
      else canvas.TextOut(Knob[1].Right,round(centerY)-round(canvas.TextHeight(inttostr(fPosition))/2),inttostr(fPosition));
     end;
  end;

 if FOrientation = scVertical then
  begin
   if (Fmax<=0) and (Fmin<0) then temp:=fmin else temp:=fmax;
   if fPosition <= round(temp/2) then
    begin
     if not Reversed then
      if (FKnob = ksArrowLeft) or (FKnob = ksArrowRight) or (FKnob = ksArrowDown) or (FKnob = ksArrowUp)
      then canvas.TextOut(round(width/2)-round(canvas.TextWidth(inttostr(fPosition))/2),Knob[1].Bottom+1,inttostr(fPosition))
      else canvas.TextOut(round(width/2)-round(canvas.TextWidth(inttostr(fPosition))/2),Knob[1].Bottom,inttostr(fPosition))
     else
      canvas.TextOut(round(width/2)-round(canvas.TextWidth(inttostr(fPosition))/2),Knob[1].Top-canvas.TextHeight(inttostr(fPosition)),inttostr(fPosition));
     end;
   if fPosition > round(temp/2) then
    begin
     if not Reversed then
      canvas.TextOut(round(width/2)-round(canvas.TextWidth(inttostr(fPosition))/2),Knob[1].Top-canvas.TextHeight(inttostr(fPosition)),inttostr(fPosition))
     else
     begin
      if (FKnob = ksArrowLeft) or (FKnob = ksArrowRight) or (FKnob = ksArrowDown) or (FKnob = ksArrowUp)
      then canvas.TextOut(round(width/2)-round(canvas.TextWidth(inttostr(fPosition))/2),Knob[1].Bottom+1,inttostr(fPosition))
      else canvas.TextOut(round(width/2)-round(canvas.TextWidth(inttostr(fPosition))/2),Knob[1].Bottom,inttostr(fPosition));
     end;
    end;
 end;
end;

procedure TSliderComponent.Paint;
var lv:integer;
begin
 CalculateDimensions;   //Berechnet die Maße des Sliders
 DrawRectangle;         //zeichnet den Hintergrund und den Track
 Knob[1]:=CalculateKnob(Pos[1]); //Maße und Mitte von Knopf1 holen
 if FOrientation = scHorizontal then DrawTrackColorsHorizontal  //Extrafarbe und SelectedFarbe
 else DrawTrackColorsVertical;
 if FOrientation = scHorizontal then DrawScaleHorizontal         //zeichnet die Skala
 else DrawScaleVertical;
 if FOrientation = scHorizontal then DrawMarkHorizontal         //zeichnet die Markierungen
 else DrawMarkVertical;
 for lv:=3 downto 1 do
  begin
   Knob[lv]:=CalculateKnob(Pos[lv]); //Position der Knöpfe ermitteln
   BuildTheKnob(lv);                 //Knöpfe zeichnen
  end;
 if FIndicator.FIndicatorVisible then DrawIndicator;  //Wertanzeige
 if Focused then     // 13.04.2020 Siro
   begin
    canvas.Pen.Width  := 1;
    canvas.Brush.Style:= bsclear;
    canvas.Pen.Color  := FFocusColor1;
    canvas.Rectangle(0+2,0+2,width-2,height-2);
    canvas.Pen.Color  := FFocusColor2;
    canvas.Pen.Style  := psdot;
    canvas.Rectangle(0+2,0+2,width-2,height-2);
   end;
  canvas.Pen.Width  := FBorderWidth;
  canvas.Pen.Color  := FBorderColor;
  canvas.Pen.Style  := pssolid;
  canvas.Brush.Style:= bssolid;
end;

procedure Register;
begin
  {$I slidercomponent_icon.lrs}
  RegisterComponents('Standard',[TSliderComponent]);
end;

end.
